package pomodoro

import (
	"git.smsvc.net/pomodoro/GoTomato/internal/shared"
	"sync"
	"time"
)

var mu sync.Mutex // to synchronize access to shared state
var timer Timer

// RunPomodoro iterates the Pomodoro work/break sessions.
func RunPomodoro() {
	mu.Lock()
	shared.Message.Ongoing = true
	shared.Message.Paused = false
	mu.Unlock()

	pomodoroConfig := shared.Message.PomodoroSettings

	for session := 1; session <= pomodoroConfig.Sessions; session++ {
		timer = timer.Init()

		shared.Message.Session = session

		shared.Message.Mode = "Work"
		if !timer.Start(pomodoroConfig.Work) {
			break
		}

		if session > pomodoroConfig.Sessions {
			shared.Message.Mode = "ShortBreak"
			if !timer.Start(pomodoroConfig.ShortBreak) {
				break
			}
		} else { // last phase, prepare for finish
			shared.Message.Mode = "LongBreak"
			if !timer.Start(pomodoroConfig.LongBreak) {
				break
			}
			shared.Message.Mode = "End"
			time.Sleep(time.Second)
		}
	}

	mu.Lock()
	shared.Message.Ongoing = false
	shared.Message.Paused = false
	mu.Unlock()

	shared.Message.Mode = "Idle"
	shared.Message.Session = 0
	shared.Message.TimeLeft = shared.Message.PomodoroSettings.Work
}

func ResetPomodoro() {
	timer.Stop()
}

func PausePomodoro() {
	mu.Lock()
	shared.Message.Paused = true
	mu.Unlock()

	timer.Pause()
}

func ResumePomodoro() {
	mu.Lock()
	shared.Message.Paused = false
	mu.Unlock()
	timer.Resume()
}

func IsPomodoroOngoing() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.Message.Ongoing
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return shared.Message.Paused
}
